/*
 *  PalmVNC Version 2.0 - A HiRes VNC client for the Palm OS
 *
 *  Copyright (C) 2003 George Styles
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 *
 */

#include <PalmOS.h>

#include "Util.h"

#include "callback.h"

#include "palmVNC.h"
#include "palmVNC_res.h"


#include "FormUtils.h"

#include <SonyCLIE.h>
#include <SonyChars.h>

void DisableJogAssist()
{
	UInt32 *flagP;

	if(!FtrGet(sonySysFtrCreator, sonySysFtrNumJogAstControlP,(UInt32*)&flagP)) {
	/* JogAssist Terminates */
		*flagP &= ~sonyJogAstControlEnable;
	}
}


void DoDebugAlert(char *s)
{
	FrmCustomAlert(DebugAlert, s, NULL, NULL);
}

void SetFormObjectVisible(FormType * frm, int TheField, Bool isVisible)
{
	UInt16 objIndex;

	objIndex = FrmGetObjectIndex(frm, TheField);
 
	if (isVisible == true)
		FrmShowObject(frm, objIndex);
	else
		FrmHideObject(frm, objIndex);
}

void SetFormFieldFromString(FormType * frm, int TheField, char * TheText)
{
	FieldPtr fieldPtr;
	VoidHand  hand;

	// Copy to form control
	fieldPtr = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, TheField));
	hand = MemHandleNew(FldGetMaxChars(fieldPtr));
	MemMove(MemHandleLock(hand), TheText, StrLen(TheText)+1);
	MemHandleUnlock(hand);
	FldSetTextHandle(fieldPtr, (Handle) hand);
}

void SetStringFromFormField(char * AString, FormType * frm, int TheField)
{
	Char *str;
	str = FldGetTextPtr(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, TheField)));
	MemSet(AString, sizeof(AString), 0);
	MemMove(AString, str, StrLen(str)+1);
}

static void SonyRemoteKeyToName(KeyCodeType AKeyCode, char *buf) // buf is 30 chars long!
 {
	switch (AKeyCode.keyCode) {
	 	case keyCodeRmcPlay:
	 		strcpy(buf, "RMC Play\0");
	 		break;
	 	case keyCodeRmcFRPlay:
	 		strcpy(buf, "RMC FR Play\0");
	 		break;
	 	case keyCodeRmcFFPlay:
	 		strcpy(buf, "RMC FF Play\0");
	 		break;
	 	case keyCodeRmcStop:
	 		strcpy(buf, "RMC Stop\0");
	 		break;
	 	case keyCodeRmcVolDown:
	 		strcpy(buf, "RMC Vol-\0");
	 		break;
	 	case keyCodeRmcVolUp:
	 		strcpy(buf, "RMC Vol+\0");
	 		break;

		default:
			strcpy(buf, "RMC Unknown\0");
			break;
		
	} //switch
 }


 void KeyCodeToKeyName(KeyCodeType AKeyCode, char *buf) // buf is 30 chars long!
 {
 	// Default value
 	char tempBuf[30];
	KeyCodeToString(AKeyCode, tempBuf);
	strcpy(buf, "(\0");
	strcat(buf, tempBuf);
	strcat(buf, ")");
	
	// Check for normal ASC chars
	if ( (AKeyCode.chr >= 33) && (AKeyCode.chr <= 127) ) {
		buf[0] = (AKeyCode.chr) & 0xFF;
		buf[1] = 0x00;
		return;
	}
	
	switch (AKeyCode.chr) {
	 	case 0:
	 		strcpy(buf, "(None Selected)\0");
	 		break;
	 	case 0x20:
	 		strcpy(buf, "Space\0");
	 		break;
		case vchrJogUp:
			strcpy(buf, "Jog Up\0");
			break;
		case vchrJogDown:
			strcpy(buf, "Jog Down\0");
			break;
		case vchrJogBack:
			strcpy(buf, "Jog Back\0");
			break;
		case vchrJogPush:
			strcpy(buf, "Jog Push\0");
			break;
		case vchrHard1:
			strcpy(buf, "Hardware 1\0");
			break;
		case vchrHard2:
			strcpy(buf, "Hardware 2\0");
			break;
		case vchrHard3:
			strcpy(buf, "Hardware 3\0");
			break;
		case vchrHard4:
			strcpy(buf, "Hardware 4\0");
			break;
			
		case vchrRmcKeyPush:
			SonyRemoteKeyToName(AKeyCode, buf);
			break;
		
	} //switch
 }

void KeyCodeToString(KeyCodeType AKeyCode, char *buf) // buf is 30 chars long!
{
	char tempBuf[30];
	// Display code, if non zero
  	if ((AKeyCode.keyCode == 0)  && (AKeyCode.chr == 0)) {
	 	strcpy(buf, "(None Selected)\0");  	
	 	return;
	}
	 
	strcpy(buf, "\0");
  	StrIToH(tempBuf, AKeyCode.chr);
  	strcat(buf, &tempBuf[4]);
  	strcat(buf, "-");
  	StrIToH(tempBuf, AKeyCode.keyCode);  	
  	strcat(buf, &tempBuf[4]);
}


static void SendCharacter(UInt16 c, Bool ctrlPressed)
{
	if (ctrlPressed)
		SendKeyEvent(0xffe3, true); // control down

	SendKeyEvent(c, true);
	SendKeyEvent(c, false);		
		
	if (ctrlPressed)
		SendKeyEvent(0xffe3, false); // control up
}


static void RunSpecialActionCode(char *action)
{
	if (strlen(action) == 0)
		return;
		
	// Check for special codes
	if (StrCaselessCompare(action, "pageup")==0)
		SendCharacter(0xff55, false);
	if (StrCaselessCompare(action, "pagedown")==0)
		SendCharacter(0xff56, false);
	if (StrCaselessCompare(action, "enter")==0)
		SendCharacter(0xff0d, false);
	if (StrCaselessCompare(action, "tab")==0)
		SendCharacter(0xff09, false);
	if (StrCaselessCompare(action, "escape")==0)
		SendCharacter(0xff1b, false);
	if (StrCaselessCompare(action, "up")==0)
		SendCharacter(0xff52, false);
	if (StrCaselessCompare(action, "down")==0)
		SendCharacter(0xff54, false);
	if (StrCaselessCompare(action, "left")==0)
		SendCharacter(0xff51, false);
	if (StrCaselessCompare(action, "right")==0)
		SendCharacter(0xff53, false);
		
	// Check for ^X which means control X
	if (strlen(action)==2) {
		if (action[0]=='^')
			SendCharacter(action[1] & 0xff, true);
	}
}


static Bool PullOutAndrunSpecialActionCode(char *action, int *index)
{
	int i, closeIndex;
	char actionCode[30];

	// Check for [[ which means send a [
	if (strlen(action)>=2)
		if (action[1]=='[') {
			SendCharacter('[', false);
			*index += 1; // move over [[
			return true; // dealt with
		} //if
	// Check we have a ] in there somewhere
	i = 0;
	closeIndex = -1;
	while (	(i<strlen(action)) && (closeIndex==-1) ) {
		if (action[i] == ']')
			closeIndex = i;
		i++;
	} //while
	
	// Quit with failed if closeIndex is still -1
	if (closeIndex == -1)
		return false;
		
	// Now copy from 1 to closeindex-1 to actionCode, and bung 0 on end
	for (i=0; i<(closeIndex-1); i++)
	{
		actionCode[i] = action[i+1];
		actionCode[i+1] = 0x00;
	}
	
	RunSpecialActionCode(actionCode);
	
	*index += (closeIndex); // Move pointer to after special code
	
	return true;
}


void RunActionSequence(char *action)
{
	int i;
	
	i=0;
	while (i<strlen(action)) {
		if (action[i] == '[') {
			// Pull out and run special character
			if (! PullOutAndrunSpecialActionCode(&action[i], &i)) { // NB Moves i on
				// It didnt pull one out, so just send the [ as it is
				SendCharacter(action[i] & 0xFF, false);
			} //
		} // if = '['
		else
			SendCharacter(action[i] & 0xFF, false);
		i++;
	} // while
}

 