/*
 *  PalmVNC Version 2.0 - A HiRes VNC client for the Palm OS
 *
 *  Copyright (C) 2003 George Styles
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 *
 */

#include <PalmOS.h>

#include "Util.h"

#include "callback.h"

#include "palmVNC.h"
#include "palmVNC_res.h"

#include "KeyMapList.h"
#include "KeyMapEdit.h"
#include "FormUtils.h"

#include "SonyChars.h"

// Pointer to KeyMapDbItem record, for use in this .c file only
KeyMapDbItem * currentEditKeyMap;

// Form reference
FormPtr keyMapEditForm;

// Actual value selected
KeyCodeType TheKeyCode;

// Set key mode flag
Bool bolSetKeyMode = false;

static void SetInSetKeyMode(Bool aValue) // Updates button
{
	bolSetKeyMode = aValue;
	SetFormObjectVisible(keyMapEditForm,  KeyMapEditSelectKeyButton, ! aValue);  
	SetFormObjectVisible(keyMapEditForm,  KeyMapEditSelectKeyLabel, aValue);  	
	// And redraw the form
	FrmDrawForm(keyMapEditForm);			
}


static void SetKeyCode(KeyCodeType ACode)
{
	char codeBuf[30] = "\0";
	char nameBuf[30] = "\0";	
	

	// For some reason, Clies give a vchrHWKeyboardPush chr code
	// with the actual char in .keyCode, so fix this
	if (ACode.chr == vchrHWKeyboardPush) {
		ACode.chr = ACode.keyCode;
		ACode.keyCode = 0;
	}
	KeyCodeToString(ACode, codeBuf);
  	KeyCodeToKeyName(ACode, nameBuf);  	  
  
  	SetFormFieldFromString(keyMapEditForm,  KeyMapEditKeyCodeField, codeBuf);
  	SetFormFieldFromString(keyMapEditForm,  KeyMapEditKeyNameField, nameBuf);

  	// And redraw the form
  	FrmDrawForm(keyMapEditForm);
  
  	// And store it
  	TheKeyCode = ACode;
}

/******************************************************************
 * KeyMapEditInitForm
 *
 *****************************************************************/
static void CopyCurrentEditKeyMapToForm() {
//	SetFormFieldFromString(keyMapEditForm, KeyMapEditNameField, currentEditKeyMap->name);
	SetFormFieldFromString(keyMapEditForm, KeyMapEditActionField, currentEditKeyMap->action);  
	SetKeyCode(currentEditKeyMap->keyCode);
//	FrmSetFocus(keyMapEditForm, FrmGetObjectIndex(keyMapEditForm, KeyMapEditNameField));
}

/******************************************************************
 * KeyMapEditInitForm
 *
 *****************************************************************/
static void CopyFormToCurrentEditKeyMap() {
//	SetStringFromFormField(currentEditKeyMap->name, keyMapEditForm, KeyMapEditNameField);
	SetStringFromFormField(currentEditKeyMap->action, keyMapEditForm, KeyMapEditActionField);	
	currentEditKeyMap->keyCode = TheKeyCode;
}


/*static void PickKeyCode()
{
	KeyCodeType AKeyCode;
	AKeyCode = DoKeyMapPickKey();
	if (AKeyCode == 0x0000) {
		return;
	}
}*/

/******************************************************************
 * KeyMApEditFormHandleEvent
 *
 * Handles events in the console form
 *****************************************************************/
static Boolean KeyMapEditFormHandleEvent(EventPtr e)
{
  Boolean handled, closeKeyMapEdit;
  char buf[50];

  CALLBACK_PROLOGUE

//	frm = FrmGetActiveForm();
	handled = false;
	closeKeyMapEdit = false;

  switch (e->eType) {
	case ctlSelectEvent:
		switch(e->data.ctlSelect.controlID) {
			case KeyMapEditSelectKeyButton:
				SetInSetKeyMode(true);
				handled = true;
				break;
			case KeyMapEditActionHelpButton:
				FrmHelp(HelpActionsString);
				handled = true;
				break;
			default:
				break;
		}
		break;

	case keyDownEvent:
		// For some reason, clies sometimes precede keyDown events with a spurious 0x12d
		if (e->data.keyDown.chr == 0x12d)
			break;
		KeyCodeToString(e->data.keyDown, buf);
		
		if (bolSetKeyMode == true)
		{
			SetKeyCode(e->data.keyDown);
			SetInSetKeyMode(false); // Updates button
			handled = true;
		}
		break; // keyDownEvent
	default:
		break;

  }

  CALLBACK_EPILOGUE

  return handled;
}


/******************************************************************
 * KeyMapEdit
 *
 * Generates the KeyMap Edit dialog
 *****************************************************************/
extern Boolean DoKeyMapEdit(KeyMapDbItem * AKeyMap) {
 	FormPtr previousForm = FrmGetActiveForm();
	UInt16 hitButton;

	keyMapEditForm = FrmInitForm(KeyMapEditForm);
	// Stash a copy of AKeyMap for use in other functions
	currentEditKeyMap = AKeyMap;
	
	FrmSetActiveForm(keyMapEditForm);
	FrmSetEventHandler(keyMapEditForm, KeyMapEditFormHandleEvent);

	/* Set default values */
	CopyCurrentEditKeyMapToForm();

	// Set SetKeyMode to false
	SetInSetKeyMode(false);
	
	// Turn off jogAssist on Sonys
//	DisableJogAssist();

//	KeySetMask(keyBitsAll);
	
	hitButton = FrmDoDialog(keyMapEditForm);
	
	if (hitButton == KeyMapEditOKButton) {
		CopyFormToCurrentEditKeyMap();
	}
	
	if (previousForm) {
		FrmSetActiveForm(previousForm);
	}
	FrmDeleteForm(keyMapEditForm);
	
	return (hitButton == KeyMapEditOKButton);
}

