/*
 *  PalmVNC Version 2.0 - A HiRes VNC client for the Palm OS
 *
 *  Copyright (C) 2003 Olivier Grardin (olivier@palmattitude.org)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 *
 */

#include <PalmOS.h>

#include "Util.h"

#include "callback.h"

#include "palmVNC.h"
#include "palmVNC_res.h"

#include "MacroList.h"

#include "MacroEdit.h"


/******************************************************************
 * MacroEditInitForm
 *
 *****************************************************************/
static void MacroEditInitForm(FormType * frm) {
	FieldPtr fieldPtr;
	VoidHand  nameH;

	fieldPtr = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, MacroEditNameField));
	nameH = MemHandleNew(FldGetMaxChars(fieldPtr));
	MemMove(MemHandleLock(nameH), currentMacro.name, StrLen(currentMacro.name)+1);
	MemHandleUnlock(nameH);
	FldSetTextHandle(fieldPtr, (Handle) nameH);
	
	FrmSetControlValue(frm, FrmGetObjectIndex(frm, MacroEditShift0Checkbox), currentMacro.shift);
	FrmSetControlValue(frm, FrmGetObjectIndex(frm, MacroEditCtrl0Checkbox), currentMacro.ctrl);
	FrmSetControlValue(frm, FrmGetObjectIndex(frm, MacroEditAlt0Checkbox), currentMacro.alt);
	
	FrmSetFocus(frm, FrmGetObjectIndex(frm, MacroEditNameField));
}


/******************************************************************
 * MacroEditFormHandleEvent
 *
 * Handles events in the console form
 *****************************************************************/
static Boolean MacroEditFormHandleEvent(EventPtr e)
{
  Boolean handled, closeMacroEdit;
  FormPtr frm;
  UInt16 field;

  CALLBACK_PROLOGUE

	frm = FrmGetActiveForm();
	handled = false;
	closeMacroEdit = false;

  switch (e->eType) {
	case ctlSelectEvent:
		switch(e->data.ctlSelect.controlID) {
			// ...

			default:
				break;
		}
		break;

	case menuEvent:
		MenuEraseStatus(NULL);
		field = FrmGetFocus(frm);

		switch(e->data.menu.itemID) {
			case EditCopy:
				FldCopy(FrmGetObjectPtr(frm, field));
				break;

			case EditPaste:
				FldPaste(FrmGetObjectPtr(frm, field));
				break;

			default:
				break;
		}
		break;

	default:
		break;

  }

  CALLBACK_EPILOGUE

  return handled;
}


/******************************************************************
 * MacroEdit
 *
 * Generates the Macro Edit dialog
 *****************************************************************/
Boolean DoMacroEdit() {
 	FormPtr previousForm = FrmGetActiveForm();
	FormPtr frm = FrmInitForm(MacroEditForm);
	UInt16 hitButton;
	Char *str;
	
	FrmSetActiveForm(frm);
	FrmSetEventHandler(frm, MacroEditFormHandleEvent);

	/* Set default values */
	MacroEditInitForm(frm);

	hitButton = FrmDoDialog(frm);
	
	if (hitButton == MacroEditOKButton) {
		str = FldGetTextPtr(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, MacroEditNameField)));
		MemSet(currentMacro.name, sizeof(currentMacro.name), 0);
		MemMove(currentMacro.name, str, StrLen(str));
		currentMacro.shift = FrmGetControlValue(frm, FrmGetObjectIndex(frm, MacroEditShift0Checkbox));
		currentMacro.ctrl = FrmGetControlValue(frm, FrmGetObjectIndex(frm, MacroEditCtrl0Checkbox));
		currentMacro.alt = FrmGetControlValue(frm, FrmGetObjectIndex(frm, MacroEditAlt0Checkbox));
	}
	
	if (previousForm) {
		FrmSetActiveForm(previousForm);
	}
	FrmDeleteForm(frm);
	
	return (hitButton == MacroEditOKButton);
}

