/******************************************************************************
 *
 * Copyright (c) 1994-2000 Palm, Inc. or its subsidiaries.
 * All rights reserved.
 *
 * File: PalmChars.h
 *
 * Release: 
 *
 * Description:
 *      Header file for all Palm Devices
 *      Contains Palm-specific vchrs
 *
 * History:
 *      01/19/01    Created by Sam Kho
 *      12/05/01    Updated by Hong Zhang
 *      12/06/01    Added the hard record button
 *      01/09/02    Merge vchrPopupBrightness (from Lighthouse), shift others.
 *      02/29/02    Add key bits for 5-way navigator.
 *
 *****************************************************************************/

#ifndef __PALMCHARS_H__
#define __PALMCHARS_H__

#include <Chars.h>

#if (vchrPalmMin != 0x0500) || (vchrPalmMax != 0x05FF)
#error "please check Palm vchr range or update the above assertion"
#endif

/*
 * NOTE: (hz) the following two codes have been updated to use correct
 * Palm vchr range. The old codes (0x1700 and 0x1701) belong to 
 * vchr range that is assigned to Sony. The change may cause some
 * compability issues. Be sure to recompile the entire system.
 */
#define vchrSilkClock           (vchrPalmMin + 0)   /* clock needs 2-phase launch */
#define vchrClock               (vchrPalmMin + 1)

#define vchrPopupBrightness     (vchrPalmMin + 2)   /* for lighthouse silk button */

#define vchrNavChange           (vchrPalmMin + 3)
#define vchrNavReserved0        (vchrPalmMin + 4)
#define vchrNavReserved1        (vchrPalmMin + 5)
#define vchrNavReserved2        (vchrPalmMin + 6)
#define vchrNavReserved3        (vchrPalmMin + 7)

/*
 * NOTE: (hz) the following key bits are specific to Palm devices.
 * We use large values to reduce potential compatibility issues.
 */
#define keyBitNavLeft           0x01000000
#define keyBitNavRight          0x02000000
#define keyBitNavSelect         0x04000000
#define keyBitNavLRS            0x07000000

#define navBitUp                0x0001
#define navBitDown              0x0002
#define navBitLeft              0x0004
#define navBitRight             0x0008
#define navBitSelect            0x0010
#define navBitsAll              0x001F

#define navChangeUp             0x0100
#define navChangeDown           0x0200
#define navChangeLeft           0x0400
#define navChangeRight          0x0800
#define navChangeSelect         0x1000
#define navChangeBitsAll        0x1F00

#define navFtrCreator           'fway'
#define navFtrVersion           0

#define navVersion              0x00010000      /* header version 1.0 */


// A macro to use for apps that support navigation using the 5-way.
// This macro will let you test whether an event was generated by the 5-way.
// For applications that treat up and down hard buttons differently than up and
// down on the 5-way, this macro can be used to determine which action to take
// if NavKeyPressed returns true for Up or Down.
//
// Usage:	if (IsFiveWayNavEvent(eventP))
//
//DOLATER - peter:
// Consider whether to ignore 5-way button presses that caused the device to
// turn on. I don't think the direction buttons can turn the device on, but the
// select button probably can. To test, look at the poweredOnKeyMask bit of
// (eventP)->data.keyDown.modifiers.
#define IsFiveWayNavEvent(eventP)																		\
	(																												\
		 ((eventP)->data.keyDown.chr == vchrPageUp ||												\
		  (eventP)->data.keyDown.chr == vchrPageDown ||												\
		  (eventP)->data.keyDown.chr == vchrNavChange)												\
	&&																												\
		 (((eventP)->data.keyDown.keyCode & (navBitsAll | navChangeBitsAll)) != 0)			\
	)


// A macro to use for apps that support navigation using the 5-way.
// By using this macro, we have consistent behavior in all our apps
// when it comes to how it handles multiple simultaneous button presses.
//
// Usage:	if (NavSelectPressed(eventP))
//
//DOLATER - peter: Choose one of these and remove all others.
//
// Only act when the select button is first pressed, and only if none of the
// direction buttons are down at the time. By ignoring repeat events for the
// select button, we ensure that the launcher will be run if the select button
// is held down.
/*
#define NavSelectPressed(eventP)																			\
	(																												\
		IsFiveWayNavEvent(eventP)																			\
	&&																												\
		(((eventP)->data.keyDown.modifiers & autoRepeatKeyMask) == 0)							\
	&&																												\
		(((eventP)->data.keyDown.keyCode & (navBitsAll   | navChangeBitsAll)) ==			\
													  (navBitSelect | navChangeSelect))					\
	)
*/
// Only act when the select button is released, and only if none of the
// direction buttons are down (or being released) at the time, and only if the select
// button has not been held down long enough to start repeating. By ignoring repeat
// events for the select button, we ensure that the launcher will be run if the select
// button is held down. By waiting until the button is released, we ensure that the
// select action is not taken before the launcher is run.
#define NavSelectPressed(eventP)																			\
	(																												\
		IsFiveWayNavEvent(eventP)																			\
	&&																												\
		(((eventP)->data.keyDown.modifiers & autoRepeatKeyMask) == 0)							\
	&&																												\
		(((eventP)->data.keyDown.keyCode & (navBitsAll | navChangeBitsAll)) ==				\
																		 navChangeSelect)						\
	)


// A macro to use for apps that support navigation using the 5-way.
// By using this macro, we have consistent behavior in all our apps
// when it comes to how it handles multiple simultaneous button presses.
// You can use this macro even if the device does not have a 5-way controller.
//
// Usage:	if (NavDirectionPressed(eventP, Left))
// 			if (NavDirectionPressed(eventP, Right))
// 			if (NavDirectionPressed(eventP, Up))			- also works without 5-way
// 			if (NavDirectionPressed(eventP, Down))			- also works without 5-way
//
//DOLATER - peter: Choose one of these and remove all others.
//
// Act when a direction is pressed, even if other directions (or select) are held down
// at the time. Repeat if the button is held down, even if other buttons are pressed.
/*
#define NavDirectionPressed(eventP, nav)																		\
	(IsFiveWayNavEvent(eventP)																				\
		? (((eventP)->data.keyDown.modifiers & autoRepeatKeyMask)								\
		 	? (((eventP)->data.keyDown.keyCode & (navChange ## nav | navBit ## nav)) ==	\
		 				(navBit ## nav))																		\
		 	: (((eventP)->data.keyDown.keyCode & (navChange ## nav | navBit ## nav)) ==	\
						(navBit ## nav | navChange ## nav)))											\
		: (((eventP)->data.keyDown.chr == vchrPageUp && navBit ## nav == navBitUp) ||		\
		 	((eventP)->data.keyDown.chr == vchrPageDown && navBit ## nav == navBitDown))	\
	)
*/
// Act only when one direction is pressed without any other direction (or select) being
// down at the time. Repeat if the button is held down, but not if other buttons are
// pressed.
#define NavDirectionPressed(eventP, nav)																		\
	(IsFiveWayNavEvent(eventP)																				\
		? (((eventP)->data.keyDown.modifiers & autoRepeatKeyMask)								\
		 	? (((eventP)->data.keyDown.keyCode & (navBitsAll | navChangeBitsAll)) ==		\
		 				(navBit ## nav))																		\
		 	: (((eventP)->data.keyDown.keyCode & (navBitsAll | navChangeBitsAll)) ==		\
						(navBit ## nav | navChange ## nav)))											\
		: (((eventP)->data.keyDown.chr == vchrPageUp && navBit ## nav == navBitUp) ||		\
		 	((eventP)->data.keyDown.chr == vchrPageDown && navBit ## nav == navBitDown))	\
	)


// A macro to use for apps that support navigation using the 5-way.
// By using this macro, we have consistent behavior in all our apps
// when it comes to how it handles multiple simultaneous button presses.
// You can use this macro even if the device does not have a 5-way controller.
//
// Usage:	if (NavKeyPressed(eventP, Select))
// 			if (NavKeyPressed(eventP, Left))
// 			if (NavKeyPressed(eventP, Right))
// 			if (NavKeyPressed(eventP, Up))			- also works without 5-way
// 			if (NavKeyPressed(eventP, Down))			- also works without 5-way
//
#define NavKeyPressed(eventP, nav)																		\
	((navBit ## nav == navBitSelect)																		\
		? NavSelectPressed(eventP)																			\
		: NavDirectionPressed(eventP, nav)																\
	)


// To match PalmOS 5.1 definition in Chars.h.
#define vchrHard5						0x0214

// The keyBit mask used for polling the 5th hard key.
#define keyBitHard5					0x2000

#define PalmFtrCreator          'PALM'
#define PalmFtrHwrBitsGroup1    0
#define HwrBitsSlider           0x0001
#define HwrBitsVoiceButton      0x0002
#define HwrBitsBuiltInKeyboard  0x0004



// The following feature should be defined on all devices that support hi-res
// screens & include NotePad 2.0 or higher.  As the OS does not yet provide
// a method for the desktop SW/conduits to query the handheld in order to determine
// the density capabilities of the device, this feature has been added.  By testing
// the presence/content of this feature, the device can properly convert any old 
// lo-density RLE data to be hi-density PNG data on the first hotsync to the device
// rather than have to do a note-by-note conversion at run time on the handheld the
// first time each old note has been opened.
#define densityFtrCreator       'dnsT'
#define densityFtrVersion       0
// one of the DensityType values defined in the Bitmap.h file should be stored in
// this feature designating the density supported by the device.



#endif /* __PALMCHARS_H__ */
