/*
 *  Copyright (C) 1998 International Computer Science Institute (ICSI)
 *
 *  Author: Vladimir Minenko, minenko@icsi.berkeley.edu
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */


#include <PalmOS.h>
#include <PalmCompatibility.h>

#include <NetMgr.h>

// Include this header so that we can use the Berkeley sockets API
#include <sys_socket.h>

#define CARD8		unsigned char
#define CARD16		unsigned short
#define CARD32		unsigned long
#define Bool		Boolean
#define False		false
#define True		true

#include "rfbproto.h"
#include "AppStdIO.h"

#include "ServerSelect.h"
#include "MacroList.h"
#include "PrefsForm.h"


// application Creator ID
#define CREATOR_ID		'PVNC'


// usable screen modes
typedef enum {
	modeUndefined = 0,
	modeHiDensity,	// Palm OS 5 high-density API
	modeSonyHires,	// Sony high-resolution API
	modeLegacy8bpp,	// Legacy modes with direct screen memory access
	modeLegacy4bpp,
	modeLegacy2bpp,
	modeHandEra330, // HandEra 330 (240x320)
	SCREEN_MODE_COUNT
} ScreenMode;

extern Char* SCREEN_MODE_DESC[SCREEN_MODE_COUNT];

extern ScreenMode VNC_screenMode;


#define Swap16IfLE(s) s
#define Swap32IfLE(l) l

#define MAX_ENCODINGS 10

#define DEFAULT_VNC_DATA_PORT		5900


#ifdef DEBUG
//#define DEBUG_VNC_CODING 	// all VNC protocol stuff, very intensive
//#define DEBUG_EVENTS		// PalmOS evetns
//#define DEBUG_FLOW 		// misc. program flow
//#define DEBUG_COLOR		// color mapping
#endif

/* defines boudaries of the Pilot viewport */
typedef struct {
  rfbRectangle	desktop;	// Dimensions of unscaled desktop
  unsigned short scaleFactor; // Scale factor for current viewport
  rfbRectangle	viewable;	// what is visible
  rfbRectangle	virtual;	// what is in the off-screen window
  rfbRectangle	remote;		// what is on the currentServer side
  rfbRectangle	zoomarea;	// specified zoom coordinates
  Boolean		zoomarea_ena; // Use specified zoom coordinates
} PRFBViewport;


extern PRFBViewport palmFB;

//extern int endianTest;

extern VncPreferenceType VNC_prefs;




/* palmVNC.c */

extern ServersDbItem currentServer;


extern void OffscreenCopyAreaFromBuffer(CARD8 *buf, short x, short y, unsigned short width, unsigned short height);
extern void PostProgressMessage(char *msgStr, unsigned short curr, unsigned short max);
extern void OffscreenDrawFilledRectangle (unsigned short colorIndex,
				short x, short y,
				unsigned short w, unsigned short h,
				int adjust);
