/*
 *  PalmVNC Version 2.0 - A HiRes VNC client for the Palm OS
 *
 *  Copyright (C) 2003 Olivier Grardin (olivier@palmattitude.org)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 *
 */


#include <PalmOS.h>

#include "CollapseUtils.h"
#include "Util.h"

#include "palmVNC.h"
#include "palmVNC_res.h"

#include "PasswordForm.h"



/******************************************************************
 * PrefsInitForm
 *
 *****************************************************************/
static void PasswordFormInit(FormType * frmP) {
}


/******************************************************************
 * PasswordFormHandleEvent
 *
 * Handles events in the password entry form
 *****************************************************************/
static Boolean PasswordFormHandleEvent(EventPtr e)
{
  Boolean handled = false;
  FormPtr frm;
  UInt16 field;

	frm = FrmGetActiveForm();
	handled = false;

	switch (e->eType) {
	
	  case frmOpenEvent: {
			    FormType * frm = FrmGetActiveForm();
					CollapseSetState(frm, collapseStateUser);
	        FrmDrawForm(frm);
	        PasswordFormInit(frm);
	        handled = true;
	      }
	      break;

		case menuEvent:
			MenuEraseStatus(NULL);
			field = FrmGetFocus(frm);
			
			switch(e->data.menu.itemID) {
				case EditCopy:
					FldCopy(FrmGetObjectPtr(frm, field));
					break;

				case EditPaste:
					FldPaste(FrmGetObjectPtr(frm, field));
					break;

				default:
					break;
			}
			break;

		default:
			break;
	}

	return handled;
}


/******************************************************************
 * GetPassword
 *
 * Generates the Password dialog
 *****************************************************************/
Boolean GetPassword(char* password, int maxlength)
{
 	FormPtr previousForm = FrmGetActiveForm();
	FormPtr frm = FrmInitForm(PasswordForm);
	UInt16 hitButton;
	FieldPtr fieldPtr;
	MemHandle passwdH;
	
	FrmSetActiveForm(frm);
	FrmSetEventHandler(frm, PasswordFormHandleEvent);

	/* Set default password to blank */
	fieldPtr = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, PasswordPasswordField));
	passwdH = MemHandleNew(maxlength);
	MemMove(MemHandleLock(passwdH), "\0", 1);
	MemHandleUnlock(passwdH);
	FldSetTextHandle(fieldPtr, (Handle)passwdH);
	FrmSetFocus(frm, FrmGetObjectIndex(frm, PasswordPasswordField));

	hitButton = FrmDoDialog(frm);
	
	if (hitButton == PasswordOKButton) {
		MemSet(password, maxlength, 0);
		if ((FldGetTextPtr(fieldPtr) != NULL) && ((StrLen(FldGetTextPtr(fieldPtr)) + 1) < maxlength)) {
			MemMove(password, FldGetTextPtr(fieldPtr), StrLen(FldGetTextPtr(fieldPtr)) + 1);
		}
		else {
			hitButton = PasswordCancelButton;
		}
	}
	
	if (previousForm) {
		FrmSetActiveForm(previousForm);
	}
	FrmDeleteForm(frm);
	
	return (hitButton == PasswordOKButton);
}
