/*
 *  PalmVNC Version 2.0 - A HiRes VNC client for the Palm OS
 *
 *  Copyright (C) 2003 Olivier Grardin (olivier@palmattitude.org)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 *
 */


#include <PalmOS.h>

#include "CollapseUtils.h"
#include "Util.h"

#include "palmVNC.h"
#include "palmVNC_res.h"

#include "PrefsForm.h"


#define MAX_SCROLLBAR_WIDTH 20
#define MIN_SCROLLBAR_WIDTH  0



/*
 * DESCRIPTION: generic color picker gadget handler
 *
 * PARAMETERS:
 *
 * RETURNED:
 */
static Boolean ColorPickerGadgetHandler(FormGadgetTypeInCallback *pGadget, UInt16 cmd, void* param) {
	UInt16 i; 
	FormType* frmP = FrmGetActiveForm();
  Boolean handled = false;
  RectangleType rect;
  IndexedColorType selectedColor;
	
//  UInt16 gadgetIndex = FrmGetObjectIndexFromPtr(frmP, pGadget);
	UInt16 gadgetIndex = frmInvalidObjectId; 
	UInt16 numObjects = FrmGetNumberOfObjects(frmP);
	for (i = 0; i < numObjects; i++) { 
	     if (FrmGetObjectPtr(frmP, i) == pGadget) { 
	          // Found it 
	          gadgetIndex = i; 
	          break; 
	    } 
	} 
	
  
  // retrieve gadget data and area if available
  if (gadgetIndex != frmInvalidObjectId) {
	  selectedColor = (IndexedColorType) FrmGetGadgetData(frmP, gadgetIndex);
	  FrmGetObjectBounds(frmP, gadgetIndex, &rect);
	}

  switch (cmd) {
  case formGadgetDrawCmd: {
  		// draw a frame around the gadget
	    WinDrawRectangleFrame(rectangleFrame, &rect);
	    // fill the gadget area with selected color
			WinPushDrawState();
			WinSetBackColor(selectedColor);
	    WinEraseRectangle(&rect, 0);
			WinPopDrawState();
	    handled = true;
     }
     break;

  case formGadgetHandleEventCmd: {
	     EventType* event = (EventType*) param;

	     if (event->eType == frmGadgetEnterEvent) {
		      // there was a tap in the gadget's bounds
					Boolean isPenDown = true;
					Int16 newPointX, newPointY;
					UInt16 ticksPerSec = SysTicksPerSecond();

					// wait until the pen goes up
					while (isPenDown){
						EvtGetPen(&newPointX, &newPointY, &isPenDown);
						SysTaskDelay(ticksPerSec / 10);
					}

					if (RctPtInRectangle (newPointX, newPointY, &rect)) {
						// the pen up was also in the gadget, popup color selection dialog
						UIPickColor(&selectedColor, NULL, UIPickColorStartPalette, NULL, NULL);
						// store selected color as gadget data
						FrmSetGadgetData(frmP, gadgetIndex, (void*) selectedColor);
						// update display
						WinPushDrawState();
						WinSetBackColor(selectedColor);
						WinEraseRectangle(&rect, 0);
						WinPopDrawState();
					}

	     }
	     else if (event->eType == frmGadgetMiscEvent) {
	     	// process custom events...
	     }
	     handled = true;
     }
     break;

  case formGadgetEraseCmd:
     //erase the frame drawn around the gadget
     WinEraseRectangleFrame(rectangleFrame, &rect);
	   // FrmHideObject takes care of  hiding the object if false is returned
     handled = false;
     break;

  case formGadgetDeleteCmd:
     handled = true;
     break;

  default:
     break;

  }//Switch(cmd)

  return handled;
}


/******************************************************************
 * PrefsInitForm
 *
 *****************************************************************/
static void PrefsFormInit(FormType * frmP) {
	FormType *prefsDialog = FrmGetActiveForm();
	static Char strWidth[3];
	Char *str;
	
	StrPrintF(strWidth, "%d", VNC_prefs.scrollbarWidth);
	FieldSetText(frmP, PrefsScrollbarWidthField, strWidth);
	
	SetCheckboxSelected(frmP, PrefsDisableProgressBarCheckbox,  VNC_prefs.disableProgressBar);
	
	// color picker gadgets
	FrmSetGadgetData(prefsDialog, FrmGetObjectIndex(prefsDialog, PrefsScrollBgColorGadget), 
		(void*) VNC_prefs.scrollBgColor);
	FrmSetGadgetData(prefsDialog, FrmGetObjectIndex(prefsDialog, PrefsScrollSliderColorGadget), 
		(void*) VNC_prefs.scrollSliderColor);

	// gadget handler	
	FrmSetGadgetHandler(prefsDialog, FrmGetObjectIndex(prefsDialog, PrefsScrollBgColorGadget), 
		ColorPickerGadgetHandler);
	FrmSetGadgetHandler(prefsDialog, FrmGetObjectIndex(prefsDialog, PrefsScrollSliderColorGadget), 
		ColorPickerGadgetHandler);
		
	// screen panning
	LstSetSelection(GetObjectPtr(frmP, PanningList), VNC_prefs.screenPanningProp);
	str = LstGetSelectionText(GetObjectPtr(frmP, PanningList), VNC_prefs.screenPanningProp);
	CtlSetLabel(GetObjectPtr(frmP, PanningPopupTrigger), str);
}


/******************************************************************
 * FixScrollbarWidth
 *
 *****************************************************************/
static void FixScrollbarWidth(Int8 *width) {
	if (*width < MIN_SCROLLBAR_WIDTH) {
		*width = MIN_SCROLLBAR_WIDTH;
	}
	else if (*width > MAX_SCROLLBAR_WIDTH) {
		*width = MAX_SCROLLBAR_WIDTH;
	}
}


/******************************************************************
 * PrefsSave
 *
 * Write the saved preferences / saved-state information.  
 * This data will be saved during a HotSync backup.
 *
 *****************************************************************/
static void PrefsSave(FormType * frmP) {
  
	Char *strWidth = FldGetTextPtr(GetObjectPtr(frmP, PrefsScrollbarWidthField));
	Boolean disableProgressBar = IsCheckboxSelected(frmP, PrefsDisableProgressBarCheckbox);

	VNC_prefs.scrollbarWidth = StrAToI(strWidth);
	VNC_prefs.disableProgressBar = disableProgressBar;
	
	VNC_prefs.scrollBgColor = (IndexedColorType) 
		FrmGetGadgetData(frmP, FrmGetObjectIndex(frmP, PrefsScrollBgColorGadget));
	VNC_prefs.scrollSliderColor = (IndexedColorType) 
		FrmGetGadgetData(frmP, FrmGetObjectIndex(frmP, PrefsScrollSliderColorGadget));
		
	VNC_prefs.screenPanningProp = LstGetSelection(GetObjectPtr(frmP, PanningList));

  PrefSetAppPreferences(
      CREATOR_ID, 0, PREFS_VERSION, 
      &VNC_prefs, sizeof(VncPreferenceType), true);
}

/******************************************************************
 * UpdateWidth
 *
 *****************************************************************/
static void UpdateWidth(FormType *frmP, Int8 delta) {
	Char *strWidth = FldGetTextPtr(GetObjectPtr(frmP, PrefsScrollbarWidthField));
	Int8 width = StrAToI(strWidth);
	Char newStrWidth[3];

	width += delta;
	FixScrollbarWidth(&width);
	
	StrPrintF(newStrWidth, "%d", width);
	FieldSetText(frmP, PrefsScrollbarWidthField, newStrWidth);
}


/*
 * DESCRIPTION:
 *
 * This routine is the event handler for the "PrefsForm" of this 
 * application.
 *
 * PARAMETERS:
 *
 * eventP
 *     a pointer to an EventType structure
 *
 * RETURNED:
 *     true if the event was handled and should not be passed to
 *     FrmHandleEvent
 */

static Boolean PrefsFormHandleEvent(EventType * eventP)
{
    Boolean handled = false;
    UInt16 controlID;

    switch (eventP->eType) 
        {
//        case menuEvent:
//            return AppDoCommand(eventP->data.menu.itemID);

        case frmOpenEvent: {
					    FormType * frm = FrmGetActiveForm();
							CollapseSetState(frm, collapseStateUser);
	            FrmDrawForm(frm);
	            PrefsFormInit(frm);
	            handled = true;
	          }
            break;
            
        case frmUpdateEvent:
          break;
            
        case ctlSelectEvent: {
	          FormType * frmP = FrmGetActiveForm();
	       		controlID = eventP->data.ctlSelect.controlID;
			      if (controlID == PrefsCancelButton) {
			        FrmReturnToForm(MainForm);
			      }
			      else if (controlID == PrefsOKButton) {
			      	PrefsSave(frmP);
			        FrmReturnToForm(MainForm);
			      }
//			      else if (controlID == PrefsUpButton) {
//						}
//			      else if (controlID == PrefsDownButton) {
//						}
					}
        	break;

        case ctlRepeatEvent: {
	          FormType * frmP = FrmGetActiveForm();
	       		controlID = eventP->data.ctlSelect.controlID;
			      if (controlID == PrefsIncRepeating) {
			      	UpdateWidth(frmP, +1);
						}
			      else if (controlID == PrefsDecRepeating) {
			      	UpdateWidth(frmP, -1);
						}
        	}
        	break;
        	
        default:
          break;
        }
    
    return handled;
}


/******************************************************************
 * PrefsForm
 *
 * Generates the PrefsForm dialog
 *****************************************************************/
void DoPrefsForm() {
	FormPtr previousForm = FrmGetActiveForm();
	FormPtr frm = FrmInitForm(PrefsForm);
	UInt16 hitButton;
	
	FrmSetActiveForm(frm);
	FrmSetEventHandler(frm, PrefsFormHandleEvent);
	
	PrefsFormInit(frm);

	hitButton = FrmDoDialog(frm);
	
	if (previousForm) {
		FrmSetActiveForm(previousForm);
	}
	FrmDeleteForm(frm);
}

