/*
 *  PalmVNC Version 2.0 - A HiRes VNC client for the Palm OS
 *
 *  Copyright (C) 2003 Olivier Grardin (olivier@palmattitude.org)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 *
 */

#include <PalmOS.h>

#include <vga.h>

#include "CollapseUtils.h"
#include "Util.h"

#include "callback.h"

#include "palmVNC.h"
#include "palmVNC_res.h"

#include "ServerEdit.h"


/******************************************************************
 * ServerDetailsFormHandleEvent
 *
 * Handles events in the console form
 *****************************************************************/
static Boolean ServerDetailsFormHandleEvent(EventPtr e)
{
  Boolean handled, closeServerDetails;
  FormPtr frm;
  UInt16 field;

  CALLBACK_PROLOGUE

	frm = FrmGetActiveForm();
	handled = false;
	closeServerDetails = false;

  switch (e->eType) {
  		case ctlSelectEvent:

			switch(e->data.ctlSelect.controlID) {
				case ServerDetailsNTPasswordSelTrigger:
					GetPassword(currentServer.passwNT, 32);
					if (StrLen(currentServer.passwNT) == 0) {
						CtlSetLabel(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerDetailsNTPasswordSelTrigger)), "-Prompt-");
					}
					else {
						CtlSetLabel(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerDetailsNTPasswordSelTrigger)), "-Assigned-");
					}
					FrmDrawForm(frm);
					handled = true;
					break;

				default:
					break;
			}
			break;

		case menuEvent:
			MenuEraseStatus(NULL);
			field = FrmGetFocus(frm);

			switch(e->data.menu.itemID) {
				case EditCopy:
					FldCopy(FrmGetObjectPtr(frm, field));
					break;

				case EditPaste:
					FldPaste(FrmGetObjectPtr(frm, field));
					break;

				default:
					break;
			}
			break;

		default:
			break;

	}

  CALLBACK_EPILOGUE
  return handled;
}


/******************************************************************
 * ServerDetails
 *
 * Generates the Server Details dialog
 *****************************************************************/
Boolean ServerDetails() {
 	FormPtr previousForm = FrmGetActiveForm();
	FormPtr frm = FrmInitForm(ServerDetailsForm);
	UInt16 hitButton;
	FieldPtr fieldPtr;
  VoidHand  beginXH, beginYH, scaleFactorH, loginNTH, basePortH;
	char buffer[32];
	char *str;
	
	CollapseSetState(frm, collapseStateUser);
	SizeForm(frm);
	if (VNC_screenMode == modeHandEra330) {
		VgaFormModify(frm, vgaFormModify160To240);
	}

	FrmSetActiveForm(frm);
	FrmSetEventHandler(frm, ServerDetailsFormHandleEvent);

	/* Set default values */
	if (StrLen(currentServer.passwNT) == 0) {
		CtlSetLabel(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerDetailsNTPasswordSelTrigger)), "-Prompt-");
	}
	else {
		CtlSetLabel(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerDetailsNTPasswordSelTrigger)), "-Assigned-");
	}

	fieldPtr = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerDetailsBeginXField));
	beginXH = MemHandleNew(FldGetMaxChars(fieldPtr));
	str = StrIToA(buffer, currentServer.beginX);
	MemMove(MemHandleLock(beginXH), str, StrLen(str)+1);
	MemHandleUnlock(beginXH);
	FldSetTextHandle(fieldPtr, (Handle)beginXH);

	fieldPtr = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerDetailsBeginYField));
	beginYH = MemHandleNew(FldGetMaxChars(fieldPtr));
	str = StrIToA(buffer, currentServer.beginY);
	MemMove(MemHandleLock (beginYH), str, StrLen(str)+1);
	MemHandleUnlock(beginYH);
	FldSetTextHandle(fieldPtr, (Handle)beginYH);

	fieldPtr = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerDetailsScaleField));
	scaleFactorH = MemHandleNew(FldGetMaxChars(fieldPtr));
	str = StrIToA(buffer, currentServer.scaleFactor);
	MemMove(MemHandleLock(scaleFactorH), str, StrLen(str)+1);
	MemHandleUnlock(scaleFactorH);
	FldSetTextHandle(fieldPtr, (Handle)scaleFactorH);

	fieldPtr = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerDetailsBasePortField));
	basePortH = MemHandleNew(FldGetMaxChars(fieldPtr)); 
	str = StrIToA(buffer, currentServer.basePort);
	MemMove(MemHandleLock(basePortH), str, StrLen(str)+1);
	MemHandleUnlock(basePortH);
	FldSetTextHandle(fieldPtr, (Handle) basePortH);

	FrmSetControlValue(frm, FrmGetObjectIndex(frm, ServerDetailsSharingCheckbox), currentServer.sharedDesktop);

	fieldPtr = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerDetailsNTNameField));
	loginNTH = MemHandleNew(FldGetMaxChars(fieldPtr));
	MemMove(MemHandleLock (loginNTH), currentServer.loginNameNT, StrLen(currentServer.loginNameNT)+1);
	MemHandleUnlock(loginNTH);
	FldSetTextHandle(fieldPtr, (Handle)loginNTH);

	hitButton = FrmDoDialog(frm);
	
	if (hitButton == ServerDetailsOKButton) {
		str = FldGetTextPtr(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerDetailsBeginXField)));
		currentServer.beginX = StrAToI(str);

		str = FldGetTextPtr(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerDetailsBeginYField)));
		currentServer.beginY = StrAToI(str);

		str = FldGetTextPtr(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerDetailsScaleField)));
		currentServer.scaleFactor = StrAToI(str);

		str = FldGetTextPtr(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerDetailsNTNameField)));
		MemSet(currentServer.loginNameNT, sizeof(currentServer.loginNameNT), 0);
		MemMove(currentServer.loginNameNT, str, StrLen(str));

		str = FldGetTextPtr(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerDetailsBasePortField)));
		currentServer.basePort = StrAToI(str);

		currentServer.sharedDesktop = FrmGetControlValue(frm, FrmGetObjectIndex(frm, ServerDetailsSharingCheckbox));
	}
	
	if (previousForm) {
		FrmSetActiveForm(previousForm);
	}
	FrmDeleteForm(frm);
	
	return (hitButton == ServerDetailsOKButton);
}


/******************************************************************
 * ServerEditFormHandleEvent
 *
 * Handles events in the console form
 *****************************************************************/
static Boolean ServerEditFormHandleEvent(EventPtr e)
{
  Boolean handled, closeServerEdit;
  FormPtr frm;
  UInt16 field;

  CALLBACK_PROLOGUE

	frm = FrmGetActiveForm();
	handled = false;
	closeServerEdit = false;

  switch (e->eType) {
	case ctlSelectEvent:
		switch(e->data.ctlSelect.controlID) {
			case ServerEditAdvancedButton:
				ServerDetails();
				handled = true;
				break;
				
			case ServerEditPasswordSelTrigger:
				GetPassword(currentServer.password, 32);
				if (StrLen(currentServer.password) == 0) {
					CtlSetLabel(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerEditPasswordSelTrigger)), "-Prompt-");
				}
				else {
					CtlSetLabel(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerEditPasswordSelTrigger)), "-Assigned-");
				}
				FrmDrawForm(frm);
				handled = true;
				break;

			default:
				break;
		}
		break;

	case menuEvent:
		MenuEraseStatus(NULL);
		field = FrmGetFocus(frm);

		switch(e->data.menu.itemID) {
			case EditCopy:
				FldCopy(FrmGetObjectPtr(frm, field));
				break;

			case EditPaste:
				FldPaste(FrmGetObjectPtr(frm, field));
				break;

			default:
				break;
		}
		break;

	default:
		break;

  }

  CALLBACK_EPILOGUE

  return handled;
}


/******************************************************************
 * ServerEdit
 *
 * Generates the Server Edit dialog
 *****************************************************************/
Boolean DoServerEdit() {
 	FormPtr previousForm = FrmGetActiveForm();
	FormPtr frm = FrmInitForm(ServerEditForm);
	UInt16 hitButton;
	FieldPtr fieldPtr;
	VoidHand  nameH, addrH, displayH;
	char buffer[32];
	char *str;
	
	CollapseSetState(frm, collapseStateUser);
	SizeForm(frm);
	if (VNC_screenMode == modeHandEra330) {
		VgaFormModify(frm, vgaFormModify160To240);
	}

	FrmSetActiveForm(frm);
	FrmSetEventHandler(frm, ServerEditFormHandleEvent);

	/* Set default values */
	fieldPtr = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerEditNameField));
	nameH = MemHandleNew(FldGetMaxChars(fieldPtr));
	MemMove(MemHandleLock(nameH), currentServer.name, StrLen(currentServer.name)+1);
	MemHandleUnlock(nameH);
	FldSetTextHandle(fieldPtr, (Handle) nameH);
	
	fieldPtr = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerEditAddressField));
	addrH = MemHandleNew(FldGetMaxChars(fieldPtr));
	MemMove(MemHandleLock(addrH),
	currentServer.address, StrLen(currentServer.address)+1);
	MemHandleUnlock(addrH);
	FldSetTextHandle(fieldPtr, (Handle) addrH);
	
	fieldPtr = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerEditDisplayField));
	displayH = MemHandleNew(FldGetMaxChars(fieldPtr));
	str = StrIToA(buffer, currentServer.displayNumber);
	MemMove(MemHandleLock(displayH), str, StrLen(str)+1);
	MemHandleUnlock(displayH);
	FldSetTextHandle(fieldPtr, (Handle) displayH);
	
	if (StrLen(currentServer.password) == 0) {
		CtlSetLabel(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerEditPasswordSelTrigger)), "-Prompt-");
	}
	else {
		CtlSetLabel(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerEditPasswordSelTrigger)), "-Assigned-");
	}
	FrmSetFocus(frm, FrmGetObjectIndex(frm, ServerEditNameField));

	hitButton = FrmDoDialog(frm);
	
	if (hitButton == ServerEditOKButton) {
		str = FldGetTextPtr(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerEditNameField)));
		MemSet(currentServer.name, sizeof(currentServer.name), 0);
		MemMove(currentServer.name, str, StrLen(str));

		str = FldGetTextPtr(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerEditAddressField)));
		MemSet(currentServer.address, sizeof(currentServer.address), 0);
		MemMove(currentServer.address, str, StrLen(str));

		str = FldGetTextPtr(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ServerEditDisplayField)));
		currentServer.displayNumber = StrAToI(str);
	}
	
	if (previousForm) {
		FrmSetActiveForm(previousForm);
	}
	FrmDeleteForm(frm);
	
	return (hitButton == ServerEditOKButton);
}

